import React, {PureComponent} from 'react';
import {
    View,
    StyleSheet,
    ActivityIndicator,
} from 'react-native';

import {
    WMButton
} from 'wm-react-native-components';

export default class SendCodeButton extends PureComponent {

    timer = null;

    /**
     * 上一次点击的时间戳
     * @type {null}
     * @private
     */
    _lastPressTime = null;

    static defaultProps = {
        enable: true,
        width: 110,
        height: 45,
        interval: 60,
        fontSize: 16,
        textColor: '#FE8D1D',
        normalBackgroundColor: '#FE8D1D',
        disabledBackgroundColor: '#999999',
        normalBorderColor:'#FE8D1D',
        disabledBorderColor:'#999999',
        normalTxtColor:'#ffffff',
        disabledTxtColor:'#ffffff',
    }

    constructor(props) {
        super(props);
        this.interval = props.interval;
        this.state = {
            showIndicator: false,
            title: props.title,
            titleColor: '#FE8D1D'
        }
    }

    setEnable = (enable) => {
        this._button && this._button.setEnable(enable);
    }

    componentWillUnmount() {
        this._clearTimer();
    }

    showIndicator = (show) => {
        this.setState({
            showIndicator: show
        })
    }

    startTimer = () => {
        if(this.isCountDowning()){
            return ;
        }
        this.interval = this.props.interval;
        this._button.setEnable(false);
        this.setState({
            title: this.interval + ' S'
        });
        this.timer = setInterval(() => {
            if (this.interval > 1) {
                this.interval --;
                this.setState({
                    title: this.interval + ' S'
                }); 
            } else {
                this.stopTimer();
            }
        }, 1000);
    }

    _clearTimer = () => {
        this.timer && clearInterval(this.timer);
        this.timer = null;
    }

    stopTimer = () => {
        this._clearTimer();
        this.setState({
            title: this.props.title
        }); 
        this._button.setEnable(true);
    }

    /**
     * 是否正在倒计时
     */
    isCountDowning = ()=>{
        return this.timer !== null;
    }

    _onpress = () => {
        let currTime = new Date().getTime();
        if(currTime - this._lastPressTime <= 2000){
            return ;
        }
        this._lastPressTime = currTime;

        if(this.state.showIndicator){
            return ;
        }
        this.props.onPress && this.props.onPress();
    }

    render() {
        // 指示器的大小。small的高度为20，large为36。
        let left = (this.props.width - 20) / 2;
        let top = (this.props.height - 20) / 2;
        return (
            <View style={[styles.container, this.props.style, {width: this.props.width, height: this.props.height}]}>
                <WMButton
                    ref={(c) => {this._button = c;}}
                    style={[styles.button,this.props.buttonStyle]}   
                    title={this.state.title}
                    fontSize={this.props.fontSize}
                    color={this.props.textColor}
                    normalTxtColor={this.props.normalTxtColor}
                    disabledTxtColor={this.props.disabledTxtColor}
                    onPress={this._onpress}
                    normalBackgroundColor={this.props.normalBackgroundColor}
                    disabledBackgroundColor={this.props.disabledBackgroundColor}
                    normalBorderColor={this.props.normalBorderColor}
                    disabledBorderColor={this.props.disabledBorderColor}
                    enable={this.props.enable}
                />
                {this.state.showIndicator ? 
                <ActivityIndicator animating={true} size='small' style={[styles.indicator, {left: left, top: top}]} /> 
                : null}
            </View>
        );
    }
}

const styles = StyleSheet.create({
    container: {
        
    },
    button: {
        flex: 1,
        borderWidth: 1,
        borderRadius: 5
    },
    indicator: {
        position: 'absolute',
    }
});